#include "stm32f10x.h"
#include "led.h"


const char fractConst[] = {'0','1','1','2','3','3','4','4','5','6','6','7','8','8','9','9'};

extern char LEDBuffer[4];
extern char digits[];

void SystemInit(void)
{
	// Enable Prefetch Buffer
	FLASH->ACR |= FLASH_ACR_PRFTBE;
    // Flash 2 wait state 
  FLASH->ACR = (FLASH->ACR & ~FLASH_ACR_LATENCY) | FLASH_ACR_LATENCY_2; 

	// Enable HSE 
	RCC->CR |= RCC_CR_HSEON;
	while (RCC->CR & RCC_CR_HSERDY == 0);
							
  	// PREDIV1 source = HSE
	RCC->CFGR &= ~RCC_CFGR_PPRE1;

	// PLLSRC = PREDIV1
	RCC->CFGR |= RCC_CFGR_PLLSRC;
							  
	// PLL = PREDIV1 * 9 => SYSCLK = 72 MHz
	RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_PLLMULL) | RCC_CFGR_PLLMULL_2 | RCC_CFGR_PLLMULL_1 | RCC_CFGR_PLLMULL_0;
							  
	// PLL enable
	RCC->CR |= RCC_CR_PLLON;
	while ((RCC->CR & RCC_CR_PLLRDY) == 0);

    RCC->CFGR = (RCC->CFGR & ~RCC_CFGR_SW) | RCC_CFGR_SW_PLL;
	while((RCC->CFGR & RCC_CFGR_SWS) != RCC_CFGR_SWS_PLL);

	// Config SysTick
	SysTick->LOAD = 5000;
	SysTick->CTRL |= SysTick_CTRL_ENABLE | SysTick_CTRL_TICKINT;
}

void reverse(char s[])
{
    int c, i, j;

    for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
        c = s[i];
        s[i] = s[j];
        s[j] = c;
    }
}

void itoa(int n, char s[])
{
    int i, sign;

    if ((sign = n) < 0)  /* record sign */
        n = -n;          /* make n positive */
    i = 0;
    do {       /* generate digits in reverse order */
        s[i++] = n % 10 + '0';   /* get next digit */
    } while ((n /= 10) > 0);     /* delete it */
    if (sign < 0)
        s[i++] = '-';
    s[i] = '\0';
    reverse(s);
} 


void SysTick_Handler()
{
		LEDEx();
}


int main()
{
		short temp, fract;
		char buffer[3];
		int i = 0, j = 0;
		char strTemp[5];
	
		InitLED();
		USART1_Init();
		
		LEDBuffer[0] = LED_CLR;
		LEDBuffer[1] = LED_CLR;
		LEDBuffer[2] = LED_CLR;
		LEDBuffer[3] = LED_CLR;
	
		while(1)
		{
				
				buffer[0] = GetChar();			
				if (buffer[0] == 0xF0)
				{
						// Read temp. data
						buffer[1] = GetChar();
						buffer[2] = GetChar();
					
						// Clear 2 most significant displays (the other two 
						// will always be overwritten)
						LEDBuffer[2] = LED_CLR;
						LEDBuffer[3] = LED_CLR;
						
						// Convert temp. register data
						temp = ((buffer[1] << 8) | buffer[2]);
						itoa((temp >> 7), strTemp);
						fract = ((temp >> 3) & 0x0F);

						// Set display buffer to display proper numbers
						LEDBuffer[0] = digits[fractConst[fract] - '0'];
						j = 1;
						for (i = strlen(strTemp) - 1; i >= 0; i--)
						{
								if (strTemp[i] == '-')
										LEDBuffer[j] = LED_MINUS;
								else
										LEDBuffer[j] = digits[strTemp[i] - '0'];
								j++;
						}
						
						LEDBuffer[1] &= ~LED_DOT;
				}
		}
}
