#include "stm32f10x.h"

//***************************************************************************************************************
// Function name: InitDisplay 
//    Initialization of SPI (only SCK and MOSI are needed) and GPIO lines for display /RST and /LOAD lines.
//
// Parameters:
//    None
//
// Return value
//    None
//
//***************************************************************************************************************
void InitSPI(void)
{
	// Enable clock for GPIOB, SPI2
	RCC->APB2ENR |= RCC_APB2ENR_IOPBEN;
	RCC->APB1ENR |= RCC_APB1ENR_SPI2EN;
	// Configure GPIOB lines
	//  - pins PB13 as floating input,	PB13 as Alternate-function push-pull output (50 MHz), pin PB12 as general purpose output (50 MHz) 
	GPIOB->CRH = GPIOB->CRH & ~(GPIO_CRH_MODE12 | GPIO_CRH_MODE13 | GPIO_CRH_MODE14 | GPIO_CRH_CNF12 | GPIO_CRH_CNF13 | GPIO_CRH_CNF14);
	GPIOB->CRH |= GPIO_CRH_MODE12_0 | GPIO_CRH_MODE12_1 | GPIO_CRH_MODE13_0 | GPIO_CRH_MODE13_1 | GPIO_CRH_CNF13_1 | GPIO_CRH_CNF14_0;

	// Set CS high
	GPIOB->ODR |= GPIO_ODR_ODR12;
	
	// Configure SPI2
	//   Master, LSB first, 8-bit frame, fPCLK/16, CLK high on idle, second edge
	//   For 72 MHz SYSCLK SPI frequency is 4.5 MHz
	SPI2->CR1 = SPI_CR1_BR_0 | SPI_CR1_BR_1 | SPI_CR1_BR_2 | SPI_CR1_MSTR | SPI_CR1_CPOL | SPI_CR1_CPHA;
	SPI2->CR2 = SPI_CR2_SSOE;

	// Enable SPI
	SPI2->CR1 |= SPI_CR1_SPE;
}


void CSLow()
{
		GPIOB->ODR &= ~(1 << 12);
}

void CSHi()
{
		GPIOB->ODR |= (1 << 12);
}



//***************************************************************************************************************
// Function name: SPI_Transmit
//    Serial data transmission.
//
// Parameters:
//    cData - byte to send
//
// Return value
//    None
//
//***************************************************************************************************************
u8 SPI_Transmit(char cData)
{ 
	while ((SPI2->SR & SPI_SR_TXE)!=SPI_SR_TXE);	
	//nLOAD_Low();
	SPI2->DR = cData;
	while ((SPI2->SR & SPI_SR_BSY)==SPI_SR_BSY);		
	return SPI2->DR;
	//nLOAD_High();
	
}



// ********************************* Platform dependent functions ********************************* 
// ********************************************** END ********************************************* 









// ********************************* Platform independent functions ********************************* 

void ReadTemp(char * data)
{
		CSLow();
		data[0] = SPI_Transmit(0);
		data[1] = SPI_Transmit(0);
		CSHi();
}

// ********************************* Platform independent functions ********************************* 
// ********************************************** END ********************************************* 
